package mccity.heroes.skills.transform;

import com.herocraftonline.heroes.Heroes;
import com.herocraftonline.heroes.api.SkillResult;
import com.herocraftonline.heroes.characters.Hero;
import com.herocraftonline.heroes.characters.skill.SkillConfigManager;
import com.herocraftonline.heroes.characters.skill.SkillType;
import com.herocraftonline.heroes.characters.skill.TargettedSkill;
import com.herocraftonline.heroes.util.Messaging;
import com.herocraftonline.heroes.util.Setting;
import com.herocraftonline.heroes.util.Util;
import org.bukkit.ChatColor;
import org.bukkit.Material;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.entity.*;
import pgDev.bukkit.DisguiseCraft.Disguise;
import pgDev.bukkit.DisguiseCraft.DisguiseCraft;
import pgDev.bukkit.DisguiseCraft.api.DisguiseCraftAPI;

import java.util.LinkedHashMap;
import java.util.Map;

public class SkillTransform extends TargettedSkill {

    private final DisguiseCraftAPI dcApi;

    private static final String PLAYER_CONFIG_KEY = "Player";
    private static final Map<Class, Disguise.MobType> mobMap = new LinkedHashMap<Class, Disguise.MobType>();
    static {
        mobMap.put(Zombie.class, Disguise.MobType.Zombie);
        mobMap.put(Skeleton.class, Disguise.MobType.Skeleton);
        mobMap.put(Slime.class, Disguise.MobType.Slime);
        mobMap.put(CaveSpider.class, Disguise.MobType.CaveSpider);
        mobMap.put(Spider.class, Disguise.MobType.Spider);
        mobMap.put(Creeper.class, Disguise.MobType.Creeper);
        mobMap.put(Silverfish.class, Disguise.MobType.Silverfish);

        mobMap.put(Enderman.class, Disguise.MobType.Enderman);
        mobMap.put(PigZombie.class, Disguise.MobType.PigZombie);
        mobMap.put(Ghast.class, Disguise.MobType.Ghast);
        mobMap.put(Blaze.class, Disguise.MobType.Blaze);
        mobMap.put(MagmaCube.class, Disguise.MobType.MagmaCube);

        mobMap.put(Pig.class, Disguise.MobType.Pig);
        mobMap.put(Sheep.class, Disguise.MobType.Sheep);
        mobMap.put(MushroomCow.class, Disguise.MobType.MushroomCow);
        mobMap.put(Cow.class, Disguise.MobType.Cow);
        mobMap.put(Chicken.class, Disguise.MobType.Chicken);
        mobMap.put(Squid.class, Disguise.MobType.Squid);

        mobMap.put(IronGolem.class, Disguise.MobType.IronGolem);
        mobMap.put(Villager.class, Disguise.MobType.Villager);
        mobMap.put(Snowman.class, Disguise.MobType.Snowman);
        mobMap.put(Ocelot.class, Disguise.MobType.Ocelot);
        mobMap.put(Wolf.class, Disguise.MobType.Wolf);

        mobMap.put(EnderDragon.class, Disguise.MobType.EnderDragon);
        mobMap.put(EnderDragonPart.class, Disguise.MobType.EnderDragon);
        mobMap.put(Giant.class, Disguise.MobType.Giant);
    }

    private String cannotTransformIntoText;
    private String transformedIntoText;

    public SkillTransform(Heroes plugin) {
        super(plugin, "Transform");
        setDescription("You transform into your target.");
        setUsage("/skill transform");
        setArgumentRange(0, 1);
        setIdentifiers("skill transform");
        setTypes(SkillType.SILENCABLE, SkillType.ILLUSION);

        dcApi = DisguiseCraft.getAPI();
    }

    @Override
    public String getDescription(Hero hero) {
        Player player = hero.getPlayer();

        StringBuilder desc = new StringBuilder();
        desc.append(getDescription());

        if (dcApi.isDisguised(player)) {
            desc.append(ChatColor.DARK_PURPLE);
            desc.append(" Current form: ");
            desc.append(disguiseToString(dcApi.getDisguise(player)));
            desc.append(ChatColor.YELLOW);
        }

        desc.append(" Max Distance:");
        desc.append(Util.formatDouble(getMaxDistanceFor(hero)));

        double cdSec = getCooldownFor(hero) / 1000.0;
        if (cdSec > 0) {
            desc.append(" CD:");
            desc.append(Util.formatDouble(cdSec));
            desc.append("s");
        }

        int mana = SkillConfigManager.getUseSetting(hero, this, Setting.MANA, 50, false);
        if (mana > 0) {
            desc.append(" M:");
            desc.append(mana);
        }

        return desc.toString();
    }

    public boolean canTransformInto(Hero hero, String creatureType) {
        return SkillConfigManager.getUseSetting(hero, this, "target." + creatureType, true);
    }

    public double getMaxDistanceFor(Hero hero) {
        return SkillConfigManager.getUseSetting(hero, this, Setting.MAX_DISTANCE, 10.0, false) +
                SkillConfigManager.getUseSetting(hero, this, Setting.MAX_DISTANCE_INCREASE, 0.1, false) * hero.getSkillLevel(this);
    }

    public int getCooldownFor(Hero hero) {
        return Math.max(0, SkillConfigManager.getUseSetting(hero, this, Setting.COOLDOWN, 100000, false) -
                SkillConfigManager.getUseSetting(hero, this, Setting.COOLDOWN_REDUCE, 1000, false) * hero.getSkillLevel(this));
    }

    public ConfigurationSection getDefaultConfig() {
        ConfigurationSection node = super.getDefaultConfig();
        node.set(Setting.REAGENT.node(), Material.EYE_OF_ENDER.getId());
        node.set(Setting.REAGENT_COST.node(), 1);
        node.set(Setting.COOLDOWN.node(), 100000);
        node.set(Setting.COOLDOWN_REDUCE.node(), 1000);
        node.set(Setting.MANA.node(), 50);
        node.set(Setting.MAX_DISTANCE.node(), 10.0);
        node.set(Setting.MAX_DISTANCE_INCREASE.node(), 0.1);

        node.set("target." + PLAYER_CONFIG_KEY, true);
        for (Disguise.MobType mobType : mobMap.values()) {
            node.set("target." + mobType.name(), true);
        }
        node.set("cannot-transform-type", "You can not transform into $1");
        node.set("transformed-into-type", "You have transformed into $1");
        return node;
    }

    @Override
    public void init() {
        cannotTransformIntoText = SkillConfigManager.getRaw(this, "cannot-transform-type", "You can not transform into $1");
        transformedIntoText = SkillConfigManager.getRaw(this, "transformed-into-type", "You have transformed into $1");
    }

    public SkillResult use(Hero hero, LivingEntity target, String args[]) {
        Player player = hero.getPlayer();
        if (player == target) return SkillResult.INVALID_TARGET_NO_MSG;

        if (target instanceof Player) {
            if (canTransformInto(hero, PLAYER_CONFIG_KEY)) {
                Player targetPlayer = (Player) target;
                Disguise dis = new Disguise(dcApi.newEntityID(), targetPlayer.getName(), null);
                disguise(player, dis);
            } else {
                Messaging.send(player, cannotTransformIntoText, PLAYER_CONFIG_KEY);
            }
        } else {
            Disguise.MobType mobType = null;
            for (Class curClass : mobMap.keySet()) {
                if (curClass.isAssignableFrom(target.getClass())) {
                    mobType = mobMap.get(curClass);
                    break;
                }
            }
            if (mobType != null) {
                if (canTransformInto(hero, mobType.name())) {
                    disguise(player, new Disguise(dcApi.newEntityID(), mobType));
                } else {
                    Messaging.send(player, cannotTransformIntoText, mobType.name());
                }
            } else {
                return SkillResult.INVALID_TARGET;
            }
        }
        broadcastExecuteText(hero, target);
        return SkillResult.NORMAL;
    }

    public void disguise(Player player, Disguise disg) {
        if (!dcApi.isDisguised(player)) {
            dcApi.disguisePlayer(player, disg);
        } else {
            dcApi.changePlayerDisguise(player, disg);
        }
        Messaging.send(player, transformedIntoText, disguiseToString(disg));
    }

    public String disguiseToString(Disguise disg) {
        return disg.mob == null ? PLAYER_CONFIG_KEY + " " + disg.data.getFirst() : disg.mob.name();
    }
}
